@@ -25,6 +25,9 @@ from dbus import UnknownMethodException, DBusException
 
 from exceptions import *
 
+import registry
+import dbus
+
 __all__ = [
            "LOCALE_TYPE",
            "LOCALE_TYPE_COLLATE",
@@ -248,7 +251,7 @@ class Accessible(BaseProxy):
                 or raises a NotImplemented error if the given interface
                 is not supported.
                 """
-                if interface in self.interfaces or interface == "org.freedesktop.atspi.Collection":
+                if interface in self.interfaces or interface == "org.a11y.atspi.Collection":
                         return self._acc_factory (self.app_name, self.acc_path, interface,
                                                                   dbus_object=self._dbus_object)
                 else:
@@ -260,6 +263,8 @@ class Accessible(BaseProxy):
 
         @property
 	def cached (self):
+                if not(registry.Registry().started):
+                        return False
                 if self._cache is not None:
 		        return (self.app_name, self.acc_path) in self._cache
                 else:
@@ -284,7 +289,7 @@ class Accessible(BaseProxy):
 		else:
                         func = self.get_dbus_method("GetApplication", dbus_interface=ATSPI_ACCESSIBLE)
 			name, path = func ()
-		return self._acc_factory (name, path, ATSPI_ACCESSIBLE)
+		return self._acc_factory (name, path, ATSPI_APPLICATION)
 
         def getAttributes(self):
                 """
@@ -328,7 +333,7 @@ class Accessible(BaseProxy):
                 an in parameter indicating which child is requested (zero-indexed).
                 @return : the 'nth' Accessible child of this object.
                 """
-                if self.cached:
+                if self.cached and not(self._cached_data.state[0] & (1 << STATE_MANAGES_DESCENDANTS)):
                         (name, path) = self._cached_data.children[index]
                 else:
                         count = Int32(self._pgetter(ATSPI_ACCESSIBLE, "ChildCount"))
@@ -337,7 +342,11 @@ class Accessible(BaseProxy):
                         func = self.get_dbus_method("GetChildAtIndex", dbus_interface=ATSPI_ACCESSIBLE)
                         (name, path) = func (index)
 
-                return self._acc_factory (name, path, ATSPI_ACCESSIBLE)
+                if (path == ATSPI_ROOT_PATH):
+                        itf = ATSPI_APPLICATION
+                else:
+                        itf = ATSPI_ACCESSIBLE
+                return self._acc_factory (name, path, itf)
 
         def getIndexInParent(self):
                 """
@@ -411,7 +420,10 @@ class Accessible(BaseProxy):
                         return _marshal_state_set(self._cached_data.state)
                 else:
                         func = self.get_dbus_method("GetState", dbus_interface=ATSPI_ACCESSIBLE)
-                        return _marshal_state_set(func())
+                        try:
+                                return _marshal_state_set(func())
+                        except LookupError:
+                                return [STATE_DEFUNCT]
 
         def isEqual(self, other):
                 """
@@ -428,7 +440,7 @@ class Accessible(BaseProxy):
                 return self.__eq__(other)
 
         def _get_childCount(self):
-                if self.cached:
+                if self.cached and not(self._cached_data.state[0] & (1 << STATE_MANAGES_DESCENDANTS)):
                         return len(self._cached_data.children)
                 else:
                         return Int32(self._pgetter(ATSPI_ACCESSIBLE, "ChildCount"))
@@ -467,7 +479,12 @@ class Accessible(BaseProxy):
                         name, path = self._cached_data.parent
                 else:
 		        name, path = self._pgetter (ATSPI_ACCESSIBLE, "Parent")
-                return self._acc_factory (name, path, ATSPI_ACCESSIBLE)
+
+                if (path == ATSPI_ROOT_PATH):
+                        itf = ATSPI_APPLICATION
+                else:
+                        itf = ATSPI_ACCESSIBLE
+                return self._acc_factory (name, path, itf)
         _parentDoc = \
                 """
                 an Accessible object which is this object's containing object.
@@ -486,4 +503,17 @@ class Accessible(BaseProxy):
                 """
         interfaces = property(fget=_get_interfaces, doc=_interfacesDoc)
 
+        def _getConstantProperty(self, interface, name):
+                if self.cached:
+                        try:
+                                getattr(self, "extraData")
+                        except (AttributeError):
+                                self.extraData = dict()
+                        try:
+                                return self.extraData[name]
+                        except (KeyError):
+                                self.extraData[name] = dbus.String(self._pgetter(interface, name))
+                                return self.extraData[name]
+                return dbus.String(self._pgetter(interface, name))
+
 #END----------------------------------------------------------------------------
