@@ -13,12 +13,14 @@
 #Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
 import string
+import gobject
 import interfaces
 from accessible import BoundingBox
 from exceptions import *
 
 from factory import AccessibleFactory
 from busutils import *
+import registry
 
 __all__ = [
                 "Event",
@@ -125,24 +127,18 @@ _klass_to_interface = {
 #------------------------------------------------------------------------------
 
 def _major_to_signal_name (name):
-        ret = string.upper(name[0])
-        for i in range(1,len(name)):
-                if (name[i] == '-'):
-                        pass
-                elif (name[i-1] == '-'):
-                        ret += string.upper(name[i])
-                else:
-                        ret += name[i]
-        return ret
-
+        def capitalize (s):
+                return s[0].upper() + s[1:]
+                
+        return ''.join ([capitalize (p) for p in name.split('-')])
+        
 def _signal_name_to_major (name):
-        ret = string.lower(name[0])
-        for i in range(1,len(name)):
-                if (name[i] == str.lower(name[i])):
-                        ret += name[i]
-                else:
-                        ret += "-" + string.lower(name[++i])
-        return ret
+        def dashize (s):
+                if s.isupper():
+                        return '-' + s.lower()
+                return s
+
+        return ''.join ([name[0].lower(),] + [dashize (c) for c in name[1:]])
 
 #------------------------------------------------------------------------------
 
@@ -157,9 +153,9 @@ def signal_spec_to_event_type (interface, name, minor):
         if klass == "focus":
                 return EventType ("focus:")
 
-        event_string = klass + ':' + major + ':'
+        event_string = klass + ':' + major
         if minor:
-                event_string += minor
+                event_string += ":" + minor
         return EventType (event_string)
 
 def event_type_to_signal_reciever(bus, factory, event_handler, event_type):
@@ -197,7 +193,8 @@ def event_type_to_signal_reciever(bus, factory, event_handler, event_type):
                 if   type.is_subtype (EventType ("object:bounds-changed")):
                         any_data = BoundingBox(*any_data)
                 elif (type.is_subtype (EventType ("object:children-changed")) or
-                      type.is_subtype (EventType ("object:property-change:parent"))):
+                      type.is_subtype (EventType ("object:property-change:accessible-parent")) or
+                      type.is_subtype (EventType ("object:active-descendant-changed"))):
                         data_name, data_path = any_data;
                         any_data = factory (data_name, data_path, interfaces.ATSPI_ACCESSIBLE)
 
@@ -208,7 +205,11 @@ def event_type_to_signal_reciever(bus, factory, event_handler, event_type):
                 # Create the source
                 source_name = sender
                 source_path = path
-                source = factory (source_name, source_path, interfaces.ATSPI_ACCESSIBLE)
+                if (path == interfaces.ATSPI_ROOT_PATH):
+                        source_itf = interfaces.ATSPI_APPLICATION
+                else:
+                        source_itf = interfaces.ATSPI_ACCESSIBLE
+                source = factory (source_name, source_path, source_itf)
 
                 event = Event (type,
                                detail1,
@@ -216,7 +217,12 @@ def event_type_to_signal_reciever(bus, factory, event_handler, event_type):
                                any_data,
                                source_application,
                                source)
-                return event_handler(event)
+                depth = gobject.main_depth()
+                r = registry.Registry()
+                if (r.async):
+                    r.enqueueEvent(event_handler, event)
+                else:
+                        return event_handler(event)
 
         return bus.add_signal_receiver(handler_wrapper, **kwargs)
 
