@@ -23,6 +23,8 @@
 
 import dbus
 import os as _os
+import Queue
+import traceback
 
 from busutils import *
 
@@ -80,6 +82,12 @@ class Registry(object):
         def __init__(self):
                 self.__dict__ = self.__shared_state
 
+                try:
+                        if (self.has_implementations):
+                                return
+                except (AttributeError):
+                        pass
+
                 self.has_implementations = False
 
                 self.device_event_register = None
@@ -116,6 +124,7 @@ class Registry(object):
                 @param app_name: D-Bus name of the application to connect to when not using the registry daemon.
                 """
 
+		self.queue = Queue.Queue()
 
                 # Set up the cache
 		cache = None
@@ -143,7 +152,9 @@ class Registry(object):
 			name = _bus_object.GetNameOwner (_ATSPI_REGISTRY_NAME)
                         self.desktop = factory (name, _ATSPI_ROOT_PATH, _ATSPI_DESKTOP)
 
+		self.async = False	# not fully supported yet
                 self.has_implementations = True
+                self.started = False
 
         def _set_default_registry (self):
                 self._set_registry (MAIN_LOOP_GLIB)
@@ -163,6 +174,7 @@ class Registry(object):
                 """
                 if not self.has_implementations:
                         self._set_default_registry ()
+                self.started = True
                 try:
                         self.main_loop.run()
                 except KeyboardInterrupt:
@@ -322,17 +334,25 @@ class Registry(object):
 
         # -------------------------------------------------------------------------------
 
+	def enqueueEvent (self, handler, event):
+		"""
+		Queue an event for later delivery.
+		"""
+		self.queue.put((handler, event))
+
         def pumpQueuedEvents (self):
                 """
-                No Longer needed all application events are asyncronous.
+                Dispatch events that have been queued.
                 """
-                pass
+		while (not(self.queue.empty())):
+			(handler, event) = self.queue.get()
+			handler(event)
 
         def flushEvents (self):
                 """
-                No Longer needed all application events are asyncronous.
+                Empty the event queue.
                 """
-                pass
+                self.queue = Queue.QUeue()
 
         # -------------------------------------------------------------------------------
 
